package jp.co.ois.sample.batch.config;

import java.io.IOException;

import javax.annotation.PostConstruct;

import org.springframework.batch.core.Job;
import org.springframework.batch.core.JobExecutionListener;
import org.springframework.batch.core.Step;
import org.springframework.batch.core.configuration.annotation.DefaultBatchConfigurer;
import org.springframework.batch.core.configuration.annotation.EnableBatchProcessing;
import org.springframework.batch.core.configuration.annotation.JobBuilderFactory;
import org.springframework.batch.core.configuration.annotation.StepBuilderFactory;
import org.springframework.batch.core.configuration.annotation.StepScope;
import org.springframework.batch.core.explore.JobExplorer;
import org.springframework.batch.core.explore.support.MapJobExplorerFactoryBean;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.core.launch.support.SimpleJobLauncher;
import org.springframework.batch.core.repository.JobRepository;
import org.springframework.batch.core.repository.support.MapJobRepositoryFactoryBean;
import org.springframework.batch.item.file.FlatFileItemWriter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.FileSystemResource;

import jp.co.ois.sample.batch.model.HogeInfo;
import jp.co.ois.sample.batch.model.HogeLine;
import jp.co.ois.sample.batch.process.HogeProcessor;
import jp.co.ois.sample.batch.process.HogeReader;
import jp.co.ois.sample.batch.process.JobListener;
import lombok.extern.slf4j.Slf4j;

@Configuration
@EnableBatchProcessing
@Slf4j
public class HogeConfig {

    @Bean
    DefaultBatchConfigurer batchConfigurer() {
        return new DefaultBatchConfigurer() {
            private JobRepository jobRepository;
            private JobExplorer jobExplorer;
            private JobLauncher jobLauncher;
            @PostConstruct
            private void init() throws Exception {
                MapJobRepositoryFactoryBean jobRepositoryFactory = new MapJobRepositoryFactoryBean();
                jobRepository = jobRepositoryFactory.getObject();
                MapJobExplorerFactoryBean jobExplorerFactory = new MapJobExplorerFactoryBean(jobRepositoryFactory);
                jobExplorer = jobExplorerFactory.getObject();
                SimpleJobLauncher simpleJobLauncher = new SimpleJobLauncher();
                simpleJobLauncher.setJobRepository(jobRepository);
                simpleJobLauncher.afterPropertiesSet();
                jobLauncher = simpleJobLauncher;
            }
            @Override
            public JobRepository getJobRepository() {
                return jobRepository;
            }
            @Override
            public JobExplorer getJobExplorer() {
                return jobExplorer;
            }
            @Override
            public JobLauncher getJobLauncher() {
                return jobLauncher;
            }
        };
    }

    @Autowired
    private JobBuilderFactory jobBuilderFactory;

    @Autowired
    private StepBuilderFactory stepBuilderFactory;

    @Value("${hoge.chunk:1}")
    private int chunk;

    @Bean
    public Job job() throws IOException {
        log.debug("called.");
        return jobBuilderFactory.get("job1")
                .incrementer(new RunIdIncrementer())
                .listener(listener())
                .start(step())
                .build();
    }

    @Bean
    public JobExecutionListener listener() {
        return new JobListener();
    }

    @Bean
    public Step step() throws IOException {
        log.debug("called.");
        return stepBuilderFactory.get("step")
                .<HogeLine, HogeInfo> chunk(chunk)
                .reader(reader(null))
                .processor(processor())
                .writer(writer(null))
                .faultTolerant()
                .skip(Exception.class)
                .skipLimit(Integer.MAX_VALUE)
                .build();
    }

    @Bean
    @StepScope
    public HogeReader reader(@Value("#{jobParameters[in]}") String fileName) throws IOException {
        return new HogeReader(fileName);
    }

    @Bean
    @StepScope
    public HogeProcessor processor()  {
        return new HogeProcessor();
    }

    @Bean
    @StepScope
    public FlatFileItemWriter<HogeInfo> writer(@Value("#{jobParameters[out]}") String filename) {
        FlatFileItemWriter<HogeInfo> writer = new FlatFileItemWriter<>();
        writer.setResource(new FileSystemResource(filename));
        writer.setLineAggregator(item -> {
            StringBuilder sb = new StringBuilder();
            sb.append(item.getName());
            sb.append("-");
            sb.append(item.getHoge());
            sb.append(item.getHoge());
            return sb.toString();
        });
        return writer;
    }
}
